/***************************************************************************/
/*                                                                         */
/*  File: game.cpp                                                         */
/*  Date: 26-08-06                                                         */
/*                                                                         */
/***************************************************************************/
/*
    Updated: xx-xx-xx
*/
/***************************************************************************/


#include <math.h>					// cosf(..) sinf(..), sqrt(..)
#include <stdio.h>					// sprintf(..)

#include <windows.h>				// GetKeyState(..)

#include <d3dx9.h>					// DirectX Header File

#pragma comment(lib, "d3d9.lib")	// DirectX Library Files
#pragma comment(lib, "d3dx9.lib")


#include "common.h"					// SafeRelease(..)

#include "collision.h"				// Collision Algorithms / Container Classes

#include "dxdbg.h"					// Various DirectX Debug

									// DirectX Device/Render Handles
extern LPDIRECT3D9			g_pD3D;			// main.cpp
extern LPDIRECT3DDEVICE9	g_pD3DDevice;	// main.cpp

extern bool					g_LMousePress;	// main.cpp
extern int					g_mousePosX;	// main.cpp
extern int					g_mousePosY;	// main.cpp

/***************************************************************************/
/*                                                                         */
/*  Globals                                                                */
/*                                                                         */
/***************************************************************************/

static bool			g_bWire			= false;  // Wire frame rendering on/off


static CPlane		g_plane			= CPlane (D3DXVECTOR3(0,0,0), D3DXVECTOR3(0,1,0));

static CLine		g_line			= CLine (D3DXVECTOR3(1,1,0), D3DXVECTOR3(0,-1,0));

static bool			g_collision		= false;  // Simple variable to check if
											  // have a collision

/***************************************************************************/
/*                                                                         */
/*  Name: KeyBoard()                                                       */
/*  Date: 03-12-05                                                         */
/*                                                                         */
/*  Used to accept keyboard input and alter some global variables - used   */
/*  for simple analysis of our code - turning wire frame rendering on/off  */
/*  adjusting variables which are used in the generation of our shape etc. */
/*                                                                         */
/***************************************************************************/
void KeyBoard()
{
	// Controlling the rendering
	if( GetKeyState('W')		& 0x80 ) g_bWire=!g_bWire;


	const float speed = 0.02f;
	D3DXVECTOR3 move = D3DXVECTOR3(0,0,0);
	
	if ( GetKeyState(VK_UP)		& 0x80 ) move.y += speed;
	if ( GetKeyState(VK_DOWN) 	& 0x80 ) move.y -= speed;

	if ( GetKeyState(VK_RIGHT) 	& 0x80 ) move.x += speed;
	if ( GetKeyState(VK_LEFT) 	& 0x80 ) move.x -= speed;

	if ( GetKeyState(0xBC) 		& 0x80 ) move.z += 0.5f*speed; // Key '>'
	if ( GetKeyState(0xBE) 		& 0x80 ) move.z -= 0.5f*speed; // Key '<'

	g_line.p0 += move;
	g_line.p1 += move;

}//End KeyBoard()


/***************************************************************************/
/*                                                                         */
/*  Name: SetCamera(..)                                                    */
/*                                                                         */
/***************************************************************************/


D3DXVECTOR3 vCamPosition;

void SetCamera()
{
	static D3DXVECTOR3 vCamRot = D3DXVECTOR3(0.1f,0,0);
	static D3DXVECTOR3 vCamRotOld = vCamRot;
	static D3DXVECTOR3 vCamDir = D3DXVECTOR3(0,4,-2);

	D3DXMATRIX mr;
	D3DXMatrixRotationYawPitchRoll(&mr, vCamRot.y, vCamRot.x, vCamRot.z);
	D3DXVec3TransformCoord(&vCamPosition, &vCamDir, &mr);

	if (g_LMousePress)
	{
		vCamRot.y = vCamRotOld.y + 0.01f*g_mousePosX;
		vCamRot.x = vCamRotOld.x + 0.01f*g_mousePosY;
	}
	else
	{
		vCamRotOld = vCamRot;
	}

    // [1] D3DTS_VIEW
    D3DXMATRIX v;
    g_pD3DDevice->SetTransform(	D3DTS_VIEW, 
								D3DXMatrixLookAtLH(&v, 
								&vCamPosition, 
                                &D3DXVECTOR3(0,0,0), 
                                &D3DXVECTOR3(0,1,0)));  
    // [2] D3DTS_PROJECTION
    D3DXMATRIX p;
    g_pD3DDevice->SetTransform( D3DTS_PROJECTION, 
								D3DXMatrixPerspectiveFovLH( &p,  
								D3DX_PI/4,  
								1.0f,
                                1.0f,500.0f));
}//End SetCamera(..)



/***************************************************************************/
/*                                                                         */
/*  Name: SetMaterial(..)                                                  */
/*                                                                         */
/***************************************************************************/

void SetMaterial()
{
      D3DMATERIAL9 matMaterial;
      D3DCOLORVALUE rgbaDiffuse  = {1.0, 1.0, 1.0, 0.0};  //RGBA
      D3DCOLORVALUE rgbaAmbient  = {1.0, 1.0, 1.0, 0.0};
      D3DCOLORVALUE rgbaSpecular = {0.0, 0.0, 0.0, 0.0};
      // emissive is for a glow effect
      D3DCOLORVALUE rgbaEmissive = {0.0, 0.0, 0.0, 0.0};

      matMaterial.Diffuse  = rgbaDiffuse;
      matMaterial.Ambient  = rgbaAmbient;
      matMaterial.Specular = rgbaSpecular;
      matMaterial.Emissive = rgbaEmissive;
      matMaterial.Power    = 100.0f;

      g_pD3DDevice->SetMaterial(&matMaterial);
}//End SetMaterial(..)



/***************************************************************************/
/*                                                                         */
/*  Name: SetLight(..)                                                     */
/*                                                                         */
/***************************************************************************/

void SetLight()
{
	//Set a light up?
    D3DLIGHT9 d3dLight;

    //Initialize the light structure.
    ZeroMemory(&d3dLight, sizeof(D3DLIGHT9));

    //Set up a white point light
    d3dLight.Type			= D3DLIGHT_POINT;

    // Actual colour of the light. (e.g. white).
    d3dLight.Diffuse.r		= 0.8f;
    d3dLight.Diffuse.g		= 0.8f;
    d3dLight.Diffuse.b		= 0.8f;

    // Colour when no light hits the object.
    d3dLight.Ambient.r		= 0.2f;
    d3dLight.Ambient.g		= 0.2f;
    d3dLight.Ambient.b		= 0.2f;

    // Shiny effect.
    d3dLight.Specular.r		= 0.1f;
    d3dLight.Specular.g     = 0.1f;
    d3dLight.Specular.b     = 0.1f;

	/*
    d3dLight.Position.x     = 0.0f;
    d3dLight.Position.y     = 0.0f;
    d3dLight.Position.z     = -10.0f;
	*/
	d3dLight.Position = vCamPosition*5.0f + D3DXVECTOR3(0,10,0);

	
    d3dLight.Attenuation0	= 5.0f; 
    d3dLight.Attenuation1	= 0.0f; 
    d3dLight.Attenuation2	= 0.0f; 
    d3dLight.Range			= 100.0f;

    g_pD3DDevice->SetLight(0, &d3dLight);
    g_pD3DDevice->LightEnable(0,TRUE);

    //Turn on lighting
	DWORD val;
	g_pD3DDevice->GetRenderState(D3DRS_LIGHTING, &val);
	if (val==FALSE)
		g_pD3DDevice->SetRenderState(D3DRS_LIGHTING, TRUE);


	g_pD3DDevice->GetRenderState(D3DRS_AMBIENT, &val);
	if (val==FALSE)
		g_pD3DDevice->SetRenderState(D3DRS_AMBIENT, 
									 D3DCOLOR_XRGB(40,40,40));
}//End SetLight(..)





/***************************************************************************/
/*                                                                         */
/*  Name: DrawShapes(..)                                                   */
/*                                                                         */
/***************************************************************************/
void DrawShape()
{
	DWORD val;
	g_pD3DDevice->GetRenderState(D3DRS_CULLMODE, &val);
	if (val != D3DCULL_NONE)
		g_pD3DDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);

	// Lets just draw a plane on the ground
	DrawTriangle(g_pD3DDevice, D3DXVECTOR3(1,0,1) ,  D3DXVECTOR3(-1,0,1) , D3DXVECTOR3(-1,0,-1));
	DrawTriangle(g_pD3DDevice, D3DXVECTOR3(-1,0,-1), D3DXVECTOR3( 1,0,-1), D3DXVECTOR3(1,0,1));

	DrawLine3D(g_pD3DDevice, g_line.p0, g_line.p1, 0xffffffff);

	// Just because where working with a ray, lets draw the line extended
	D3DXVECTOR3 rDir = g_line.p1 - g_line.p0;
	D3DXVec3Normalize(&rDir, &rDir);

	DrawLine3D(g_pD3DDevice, g_line.p1, g_line.p1 + rDir*10.0f, 0xff0fffff);


}//End DrawShape(..)


/***************************************************************************/
/*                                                                         */
/* GameLoop()                                                              */
/* Our main render loop, which gets called over and over agian to do our   */
/* drawing...clears the screen, draws the data, then presents it..         */
/*                                                                         */
/***************************************************************************/
void GameLoop()
{
	if(!g_pD3DDevice)return;

		
	SetMaterial();

	SetLight();

	SetCamera();

	KeyBoard();												// Get keyboard input

	// Clear the back buffer to a blue color
	g_pD3DDevice->Clear( 0, 
						 NULL, 
						 D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, 
						 D3DCOLOR_XRGB(0,20,155), 
						 1.0f, 
						 0 );


	g_pD3DDevice->BeginScene();

	DrawShape();											// Create & Render Shape


	D3DXVECTOR3 hit;
	D3DXVECTOR3 closestPoint;

	D3DXVECTOR3 rayDir = g_line.p1 - g_line.p0;
	D3DXVec3Normalize(&rayDir, &rayDir);

	g_collision = RayPlane( g_line.p0,
							rayDir,
							g_plane.p,
							g_plane.n,
							&hit,
							&closestPoint);
							

	static debugtxt abc(g_pD3DDevice);
	abc.printf("Cursor Keys Move Object", 10, 30);
	abc.printf("Keyboard < & > To Move Object Into Z", 10, 45);
	abc.printf("LeftMouse Button and Drag To Move Cam", 10, 60);

	static debugsphere ball(g_pD3DDevice);
	static debugsphere ball2(g_pD3DDevice, 0x0000afff);

	if (g_collision)
	{
		abc.printf("***Collision***", 10, 75);

		ball.Draw(g_pD3DDevice, hit.x, hit.y, hit.z, 0.03f);

		ball2.Draw(g_pD3DDevice, closestPoint.x, closestPoint.y, closestPoint.z, 0.02f);
	}

	// Debug information - display what sort of Frames Per Second Where Getting
	FPSDisplay(g_pD3DDevice);

	g_pD3DDevice->EndScene();
	
	// After rendering the scene we display it.
	g_pD3DDevice->Present( NULL, NULL, NULL, NULL );


}//End GameLoop()

/***************************************************************************/

bool Create()
{
	return true;
}// End Create(..)

/***************************************************************************/

bool Release()
{
	return true;
}// End Release(..)